/*
 * Created on Jun 13, 2003
 */
package core;

import java.io.ByteArrayOutputStream;
import java.math.BigInteger;
import java.security.interfaces.RSAKey;
import java.security.interfaces.RSAPrivateKey;
import java.security.interfaces.RSAPublicKey;

import org.bouncycastle.crypto.AsymmetricBlockCipher;
import org.bouncycastle.crypto.engines.RSAEngine;
import org.bouncycastle.crypto.params.RSAKeyParameters;

/**
 * This library realise the RSA/ECB/PKCS#1 encoding if given a provider of the RSA cipher
 * which does not support any mode  
 * @author Xavier Didelot
 */
public class RSALib {

	/**
	 * Encrypt a given array of bytes with a given RSAKey 
	 * @param in array of bytes that have to be encrypted
	 * @param k key used for RSA encryption
	 * @return the array of bytes after encryption
	 * @throws Exception
	 */
	static public byte[] RSAenc(byte[] in,RSAKey k) throws Exception 
	{ 
	ByteArrayOutputStream out = new ByteArrayOutputStream(); 
	AsymmetricBlockCipher eng = new RSAEngine(); 
	BigInteger bi;
	if (k.getClass().getName().endsWith("RSAPublicKey")) bi=((RSAPublicKey)k).getPublicExponent();
	else bi=((RSAPrivateKey)k).getPrivateExponent();
	eng.init(true,new RSAKeyParameters(false,k.getModulus(),bi)); 
	int s = eng.getInputBlockSize(); 
	int r=0; 
	for (int t=0; t<in.length; t+= s) 
	{ 
		System.out.print("*");
	if (in.length - t <= s) {r = in.length-t;} else {r=s;} 
	out.write(eng.processBlock(in,t,r)); 
	} 
	System.out.println("");
	out.flush(); 
	out.close(); 
	return(out.toByteArray()); 
	} 
 
	/**
	 * Decrypt a given array of bytes with a given RSAKey 
	 * @param in array of bytes that have to be decrypted
	 * @param k key used for RSA decryption
	 * @return the array of bytes after decryption
	 * @throws Exception
	 */
	static public byte[] RSAdec(byte[] in,RSAKey k) throws Exception 
	{ 
	AsymmetricBlockCipher eng = new RSAEngine(); 
	ByteArrayOutputStream out = new ByteArrayOutputStream(); 
	BigInteger bi;
	if (k.getClass().getName().endsWith("RSAPublicKey")) bi=((RSAPublicKey)k).getPublicExponent();
	else bi=((RSAPrivateKey)k).getPrivateExponent();
	eng.init(false,new RSAKeyParameters(true,k.getModulus(),bi)); 
	int s = eng.getInputBlockSize();
	int r=0; 
	for (int t=0; t<in.length; t+= s) 
	{ 
	if (in.length - t <= s) {r = in.length-t;} else {r=s;} 
	out.write(eng.processBlock(in,t,r)); 
	} 
	out.flush(); 
	out.close(); 
	return(out.toByteArray()); 
	} 
	
}

