package protocols;

import core.*;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.lang.reflect.Array;
import java.io.ObjectOutputStream;
import java.io.ByteArrayOutputStream;
import java.math.BigInteger;

public class ANTOINE extends Protocol {
   
   public ANTOINE (ObjectOutputStream oos) {
      super("core.MessageT","core.MessageKeyT","core.NonceT");
      this.oos=oos;
      }
   
   public void run() {
      try {
         
         println("creating nonce carddata");
         createNewNonce("carddata");
         println("creating nonce choice");
         createNewNonce("choice");
         println("creating nonce request");
         createNewNonce("request");
         println("creating nonce acknowledgement");
         createNewNonce("acknowledgement");
         
         println("Building message m1");
         Message m1 =message();
         m1.addMessage(message(agentFromId("A")));
         m0=message();
         storeM0();
         m0.addMessage(message(agentFromId("C")));
         println("Encrypting m0");
         m0.encrypt("RSA", getPrivateKey(nameFromId("A")));
         m0=message(restoreM0(),m0);
         m0.addMessage(message(nonceFromId("request")));
         println("Encrypting m0");
         m0.encrypt("RSA", getPublicKey(nameFromId("C")));
         m1.addMessage(m0);
         println("Sending message m1");
         put("C", m1.getString(),4001);
         
         println("waiting for m2");
         Message m2 = message(get("C"));
         checkTag(m2,"PNN)");
         println("Decrypting m2");
         m2.decrypt("RSA", getPrivateKey(nameFromId("A")));
         checkTag(m2,"NN");
         addParts(m2.getAllFields(),2);
         m0=getPart();
         setValueToNonce(m0.getString(),"request");
         m0=getPart();
         setValueToNonce(m0.getString(),"answer");
         
         
         println("Building message m3");
         Message m3 =message();
         m3.addMessage(message(nonceFromId("answer")));
         m3.addMessage(message(nonceFromId("choice")));
         m0=message();
         m0.addMessage(message(agentFromId("B")));
         storeM0();
         m0.setStringAddr(IPFromId("B"));
         m0=message(restoreM0(),m0);
         println("Encrypting m0");
         m0.encrypt("RSA", getPrivateKey(nameFromId("A")));
         m3.addMessage(m0);
         m0=message();
         m0.addMessage(message(nonceFromId("carddata")));
         println("Encrypting m0");
         m0.encrypt("RSA", getPublicKey(nameFromId("B")));
         println("Encrypting m0");
         m0.encrypt("RSA", getPrivateKey(nameFromId("A")));
         m3.addMessage(m0);
         println("Encrypting m3");
         m3.encrypt("RSA", getPublicKey(nameFromId("C")));
         println("Sending message m3");
         put("C", m3.getString());
         
         println("waiting for m8");
         Message m8 = message(get("C"));
         checkTag(m8,"PSA)SPN))N)");
         println("Decrypting m8");
         m8.decrypt("RSA", getPrivateKey(nameFromId("A")));
         checkTag(m8,"SA)SPN))N");
         addParts(m8.getAllFields(),3);
         m0=getPart();
         println("Decrypting m0");
         m0.decrypt("RSA", getPublicKey(nameFromId("C")));
         checkTag(m0,"A");
         addParts(m0.getAllFields(),1);
         m0=getPart();
         setNameToId(m0.getString(),"A");
         
         m0=getPart();
         println("Decrypting m0");
         m0.decrypt("RSA", getPublicKey(nameFromId("B")));
         checkTag(m0,"PN)");
         addParts(m0.getAllFields(),1);
         m0=getPart();
         println("Decrypting m0");
         m0.decrypt("RSA", getPrivateKey(nameFromId("A")));
         checkTag(m0,"N");
         addParts(m0.getAllFields(),1);
         m0=getPart();
         setValueToNonce(m0.getString(),"carddata");
         
         
         m0=getPart();
         setValueToNonce(m0.getString(),"receipt");
         
         
         println("Building message m9");
         Message m9 =message();
         m9.addMessage(message(nonceFromId("receipt")));
         m9.addMessage(message(nonceFromId("acknowledgement")));
         println("Encrypting m9");
         m9.encrypt("RSA", getPublicKey(nameFromId("C")));
         println("Sending message m9");
         put("C", m9.getString());
         
         writeObject(nonceFromId("choice"));
         writeObject(nonceFromId("receipt"));
         
         
         } catch (Exception e) {e.printStackTrace();System.exit(0);}
      }
   
   public static void subMain(String[] args,ObjectOutputStream oos) {
      ANTOINE A = new ANTOINE(oos);
      args=A.getArgs(args);
      A.start();
      }
   
   public String[] getArgs(String[] args) {
      try {
         if (args.length != 7) {
            args = (String[]) Array.newInstance("".getClass(), 7);
            BufferedReader in2 =
            new BufferedReader(new InputStreamReader(System.in));
            println("Type the KeyStore file to use  [keystore]");
            args[0]=in2.readLine();
            if (args[0].compareTo("")==0) args[0]="keystore";
            println("Type the password to access it [duitama]");
            args[1]=in2.readLine();
            if (args[1].compareTo("")==0) args[1]="duitama";
            println("Type the DNS name of agent B [localhost]");
            args[2]=in2.readLine();
            if (args[2].compareTo("")==0) args[2]="localhost";
            println("Type the DNS name of agent C [localhost]");
            args[3]=in2.readLine();
            if (args[3].compareTo("")==0) args[3]="localhost";
            println("Type the name of agent A [A]");
            args[4]=in2.readLine();
            if (args[4].compareTo("")==0) args[4]="A";
            println("Type the name of agent B [B]");
            args[5]=in2.readLine();
            if (args[5].compareTo("")==0) args[5]="B";
            println("Type the name of agent C [C]");
            args[6]=in2.readLine();
            if (args[6].compareTo("")==0) args[6]="C";
            }
         setKS(args[0],args[1]);
         setIPToId(args[2],"B");
         setIPToId(args[3],"C");
         setNameToId(args[4],"A");
         setNameToId(args[5],"B");
         setNameToId(args[6],"C");
         } catch (Exception e) {e.printStackTrace();}
      return args;
      }
   
   public static void main(String[] args) {
      ObjectOutputStream oos=null;
      try {
         ByteArrayOutputStream os = new ByteArrayOutputStream();
         oos=new ObjectOutputStream(os);
         PrintOOS p=new PrintOOS("ANTOINE",oos,os);
         p.start();}
      catch (Exception e) {e.printStackTrace();}
      subMain(args,oos);
      }
   }